@@ -15,7 +15,7 @@ from pyqtgraph.Qt import QtGui, QtCore
 import pyqtgraph as pg
 
 
-app = QtGui.QApplication([])
+app = pg.mkQApp("Arrow Example")
 
 w = QtGui.QMainWindow()
 cw = pg.GraphicsLayoutWidget()
@@ -50,8 +50,5 @@ p2.addItem(a)
 anim = a.makeAnimation(loop=-1)
 anim.start()
 
-## Start Qt event loop unless running in interactive mode or using pyside.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
