@@ -2,7 +2,7 @@
 """
 This example demonstrates a very basic use of flowcharts: filter data,
 displaying both the input and output of the filter. The behavior of
-he filter can be reprogrammed by the user.
+the filter can be reprogrammed by the user.
 
 Basic steps are:
   - create a flowchart and two plots
@@ -19,7 +19,7 @@ import pyqtgraph as pg
 import numpy as np
 import pyqtgraph.metaarray as metaarray
 
-app = QtGui.QApplication([])
+app = pg.mkQApp("Flowchart Example")
 
 ## Create main window with grid layout
 win = QtGui.QMainWindow()
@@ -75,10 +75,5 @@ fc.connectTerminals(fc['dataIn'], pw1Node['In'])
 fc.connectTerminals(fNode['Out'], pw2Node['In'])
 fc.connectTerminals(fNode['Out'], fc['dataOut'])
 
-
-
-## Start Qt event loop unless running in interactive mode or using pyside.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
