@@ -6,10 +6,10 @@ Very basic 3D graphics example; create a view widget and add a few items.
 ## Add path to library (just for examples; you do not need this)
 import initExample
 
-from pyqtgraph.Qt import QtCore, QtGui
+from pyqtgraph.Qt import QtCore, QtGui, mkQApp
 import pyqtgraph.opengl as gl
 
-app = QtGui.QApplication([])
+app = mkQApp("GLViewWidget Example")
 w = gl.GLViewWidget()
 w.opts['distance'] = 20
 w.show()
@@ -27,8 +27,5 @@ ax2.setParentItem(b)
 
 b.translate(1,1,1)
 
-## Start Qt event loop unless running in interactive mode.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
