@@ -12,12 +12,11 @@ from pyqtgraph.Qt import QtGui, QtCore
 import numpy as np
 import pyqtgraph as pg
 
-#QtGui.QApplication.setGraphicsSystem('raster')
-app = QtGui.QApplication([])
+app = pg.mkQApp("Plotting Example")
 #mw = QtGui.QMainWindow()
 #mw.resize(800,800)
 
-win = pg.GraphicsWindow(title="Basic plotting examples")
+win = pg.GraphicsLayoutWidget(show=True, title="Basic plotting examples")
 win.resize(1000,600)
 win.setWindowTitle('pyqtgraph example: Plotting')
 
@@ -28,8 +27,8 @@ p1 = win.addPlot(title="Basic array plotting", y=np.random.normal(size=100))
 
 p2 = win.addPlot(title="Multiple curves")
 p2.plot(np.random.normal(size=100), pen=(255,0,0), name="Red curve")
-p2.plot(np.random.normal(size=110)+5, pen=(0,255,0), name="Blue curve")
-p2.plot(np.random.normal(size=120)+10, pen=(0,0,255), name="Green curve")
+p2.plot(np.random.normal(size=110)+5, pen=(0,255,0), name="Green curve")
+p2.plot(np.random.normal(size=120)+10, pen=(0,0,255), name="Blue curve")
 
 p3 = win.addPlot(title="Drawing with points")
 p3.plot(np.random.normal(size=100), pen=(200,200,200), symbolBrush=(255,0,0), symbolPen='w')
@@ -96,8 +95,5 @@ lr.sigRegionChanged.connect(updatePlot)
 p9.sigXRangeChanged.connect(updateRegion)
 updatePlot()
 
-## Start Qt event loop unless running in interactive mode or using pyside.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
