@@ -14,7 +14,7 @@ import numpy as np
 from collections import namedtuple
 from itertools import chain
 
-app = QtGui.QApplication([])
+app = pg.mkQApp("Scatter Plot Item Example") 
 mw = QtGui.QMainWindow()
 mw.resize(800,800)
 view = pg.GraphicsLayoutWidget()  ## GraphicsView with GraphicsLayout inserted by default
@@ -134,9 +134,5 @@ s4.addPoints(
 w4.addItem(s4)
 s4.sigClicked.connect(clicked)
 
-
-## Start Qt event loop unless running in interactive mode.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
