@@ -9,7 +9,7 @@ import pyqtgraph as pg
 from pyqtgraph.Qt import QtCore, QtGui
 import numpy as np
 
-app = QtGui.QApplication([])
+app = pg.mkQApp("Table Widget Example")
 
 w = pg.TableWidget()
 w.show()
@@ -26,9 +26,5 @@ data = np.array([
     
 w.setData(data)
 
-
-## Start Qt event loop unless running in interactive mode or using pyside.
 if __name__ == '__main__':
-    import sys
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
+    pg.mkQApp().exec_()
