@@ -1,13 +1,12 @@
+import numpy as np
+import collections
+import sys, os
 import pyqtgraph as pg
 from pyqtgraph.Qt import QtGui, QtCore
 from pyqtgraph.parametertree import Parameter, ParameterTree
 from pyqtgraph.parametertree import types as pTypes
 import pyqtgraph.configfile
-import numpy as np
-import user
-import collections
-import sys, os
-
+from pyqtgraph.python2_3 import xrange
 
 
 class RelativityGUI(QtGui.QWidget):
@@ -132,7 +131,7 @@ class RelativityGUI(QtGui.QWidget):
     def setAnimation(self, a):
         if a:
             self.lastAnimTime = pg.ptime.time()
-            self.animTimer.start(self.animDt*1000)
+            self.animTimer.start(int(self.animDt*1000))
         else:
             self.animTimer.stop()
             
@@ -160,17 +159,21 @@ class RelativityGUI(QtGui.QWidget):
         self.setAnimation(self.params['Animate'])
         
     def save(self):
-        fn = str(pg.QtGui.QFileDialog.getSaveFileName(self, "Save State..", "untitled.cfg", "Config Files (*.cfg)"))
-        if fn == '':
+        filename = pg.QtGui.QFileDialog.getSaveFileName(self, "Save State..", "untitled.cfg", "Config Files (*.cfg)")
+        if isinstance(filename, tuple):
+            filename = filename[0]  # Qt4/5 API difference
+        if filename == '':
             return
         state = self.params.saveState()
-        pg.configfile.writeConfigFile(state, fn) 
+        pg.configfile.writeConfigFile(state, str(filename)) 
         
     def load(self):
-        fn = str(pg.QtGui.QFileDialog.getOpenFileName(self, "Save State..", "", "Config Files (*.cfg)"))
-        if fn == '':
+        filename = pg.QtGui.QFileDialog.getOpenFileName(self, "Save State..", "", "Config Files (*.cfg)")
+        if isinstance(filename, tuple):
+            filename = filename[0]  # Qt4/5 API difference
+        if filename == '':
             return
-        state = pg.configfile.readConfigFile(fn) 
+        state = pg.configfile.readConfigFile(str(filename)) 
         self.loadState(state)
         
     def loadPreset(self, param, preset):
@@ -247,7 +250,7 @@ class GridParam(pTypes.GroupParameter):
         template = self.param('ClockTemplate')
         spacing = self['Spacing']
         for i in range(self['Number of Clocks']):
-            c = template.buildClocks().values()[0]
+            c = list(template.buildClocks().values())[0]
             c.x0 += i * spacing
             clocks[self.name() + '%02d' % i] = c
         return clocks
@@ -502,7 +505,7 @@ class Simulation:
         
     def run(self):
         nPts = int(self.duration/self.dt)+1
-        for cl in self.clocks.itervalues():
+        for cl in self.clocks.values():
             cl.init(nPts)
             
         if self.ref is None:
@@ -514,7 +517,7 @@ class Simulation:
         clocks = self.clocks
         dt = self.dt
         tVals = np.linspace(0, dt*(nPts-1), nPts)
-        for cl in self.clocks.itervalues():
+        for cl in self.clocks.values():
             for i in xrange(1,nPts):
                 nextT = tVals[i]
                 while True:
@@ -549,7 +552,7 @@ class Simulation:
         
         ## make sure reference clock is not present in the list of clocks--this will be handled separately.
         clocks = clocks.copy()
-        for k,v in clocks.iteritems():
+        for k,v in clocks.items():
             if v is ref:
                 del clocks[k]
                 break
@@ -586,7 +589,7 @@ class Simulation:
             
             
             ## update all other clocks
-            for cl in clocks.itervalues():
+            for cl in clocks.values():
                 while True:
                     g = cl.acceleration()
                     tau1, tau2 = cl.accelLimits()
@@ -635,7 +638,7 @@ class Simulation:
         
     def plot(self, plot):
         plot.clear()
-        for cl in self.clocks.itervalues():
+        for cl in self.clocks.values():
             c, p = cl.getCurve()
             plot.addItem(c)
             plot.addItem(p)
@@ -651,15 +654,6 @@ class Animation(pg.ItemGroup):
             item = ClockItem(cl)
             self.addItem(item)
             self.items[name] = item
-            
-        #self.timer = timer
-        #self.timer.timeout.connect(self.step)
-        
-    #def run(self, run):
-        #if not run:
-            #self.timer.stop()
-        #else:
-            #self.timer.start(self.dt)
         
     def restart(self):
         for cl in self.items.values():
@@ -675,7 +669,8 @@ class ClockItem(pg.ItemGroup):
         pg.ItemGroup.__init__(self)
         self.size = clock.size
         self.item = QtGui.QGraphicsEllipseItem(QtCore.QRectF(0, 0, self.size, self.size))
-        self.item.translate(-self.size*0.5, -self.size*0.5)
+        tr = QtGui.QTransform.fromTranslate(-self.size*0.5, -self.size*0.5)
+        self.item.setTransform(tr)
         self.item.setPen(pg.mkPen(100,100,100))
         self.item.setBrush(clock.brush)
         self.hand = QtGui.QGraphicsLineItem(0, 0, 0, self.size*0.5)
@@ -719,19 +714,19 @@ class ClockItem(pg.ItemGroup):
         t = data['pt'][self.i]
         self.hand.setRotation(-0.25 * t * 360.)
         
-        self.resetTransform()
         v = data['v'][self.i]
         gam = (1.0 - v**2)**0.5
-        self.scale(gam, 1.0)
+        self.setTransform(QtGui.QTransform.fromScale(gam, 1.0))
         
         f = data['f'][self.i]
-        self.flare.resetTransform()
+        tr = QtGui.QTransform()
         if f < 0:
-            self.flare.translate(self.size*0.4, 0)
+            tr.translate(self.size*0.4, 0)
         else:
-            self.flare.translate(-self.size*0.4, 0)
+            tr.translate(-self.size*0.4, 0)
         
-        self.flare.scale(-f * (0.5+np.random.random()*0.1), 1.0)
+        tr.scale(-f * (0.5+np.random.random()*0.1), 1.0)
+        self.flare.setTransform(tr)
         
         if self._spaceline is not None:
             self._spaceline.setPos(pg.Point(data['x'][self.i], data['t'][self.i]))
@@ -755,7 +750,7 @@ class ClockItem(pg.ItemGroup):
         #pass
 
 if __name__ == '__main__':
-    pg.mkQApp()
+    app = pg.mkQApp()
     #import pyqtgraph.console
     #cw = pyqtgraph.console.ConsoleWidget()
     #cw.show()
@@ -764,10 +759,5 @@ if __name__ == '__main__':
     win.setWindowTitle("Relativity!")
     win.show()
     win.resize(1100,700)
-    
-    if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
-        QtGui.QApplication.instance().exec_()
-    
-    
-    #win.params.param('Objects').restoreState(state, removeChildren=False)
 
+    app.exec_()
