@@ -1,18 +1,23 @@
+# -*- coding: utf-8 -*-
 from ..Qt import QtCore, QtGui
 
 from .DockDrop import *
 from ..widgets.VerticalLabel import VerticalLabel
 from ..python2_3 import asUnicode
 
+
 class Dock(QtGui.QWidget, DockDrop):
-    
+
     sigStretchChanged = QtCore.Signal()
-    
-    def __init__(self, name, area=None, size=(10, 10), widget=None, hideTitle=False, autoOrientation=True, closable=False):
+    sigClosed = QtCore.Signal(object)
+
+    def __init__(self, name, area=None, size=(10, 10), widget=None, hideTitle=False, autoOrientation=True, closable=False, fontSize="12px"):
         QtGui.QWidget.__init__(self)
         DockDrop.__init__(self)
+        self._container = None
+        self._name = name
         self.area = area
-        self.label = DockLabel(name, self, closable)
+        self.label = DockLabel(name, self, closable, fontSize)
         if closable:
             self.label.sigCloseClicked.connect(self.close)
         self.labelHidden = False
@@ -33,6 +38,7 @@ class Dock(QtGui.QWidget, DockDrop):
         self.widgetArea.setLayout(self.layout)
         self.widgetArea.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
         self.widgets = []
+        self._container = None
         self.currentRow = 0
         #self.titlePos = 'top'
         self.raiseOverlay()
@@ -64,9 +70,9 @@ class Dock(QtGui.QWidget, DockDrop):
         }"""
         self.setAutoFillBackground(False)
         self.widgetArea.setStyleSheet(self.hStyle)
-        
+
         self.setStretch(*size)
-        
+
         if widget is not None:
             self.addWidget(widget)
 
@@ -78,34 +84,22 @@ class Dock(QtGui.QWidget, DockDrop):
             return ['dock']
         else:
             return name == 'dock'
-        
+
     def setStretch(self, x=None, y=None):
         """
         Set the 'target' size for this Dock.
         The actual size will be determined by comparing this Dock's
         stretch value to the rest of the docks it shares space with.
         """
-        #print "setStretch", self, x, y
-        #self._stretch = (x, y)
         if x is None:
             x = 0
         if y is None:
             y = 0
-        #policy = self.sizePolicy()
-        #policy.setHorizontalStretch(x)
-        #policy.setVerticalStretch(y)
-        #self.setSizePolicy(policy)
         self._stretch = (x, y)
         self.sigStretchChanged.emit()
-        #print "setStretch", self, x, y, self.stretch()
         
     def stretch(self):
-        #policy = self.sizePolicy()
-        #return policy.horizontalStretch(), policy.verticalStretch()
         return self._stretch
-        
-    #def stretch(self):
-        #return self._stretch
 
     def hideTitleBar(self):
         """
@@ -117,7 +111,7 @@ class Dock(QtGui.QWidget, DockDrop):
         if 'center' in self.allowedAreas:
             self.allowedAreas.remove('center')
         self.updateStyle()
-        
+
     def showTitleBar(self):
         """
         Show the title bar for this Dock.
@@ -126,7 +120,19 @@ class Dock(QtGui.QWidget, DockDrop):
         self.labelHidden = False
         self.allowedAreas.add('center')
         self.updateStyle()
-        
+
+    def title(self):
+        """
+        Gets the text displayed in the title bar for this dock.
+        """
+        return asUnicode(self.label.text())
+
+    def setTitle(self, text):
+        """
+        Sets the text displayed in title bar for this Dock.
+        """
+        self.label.setText(text)
+
     def setOrientation(self, o='auto', force=False):
         """
         Sets the orientation of the title bar for this Dock.
@@ -134,7 +140,12 @@ class Dock(QtGui.QWidget, DockDrop):
         By default ('auto'), the orientation is determined
         based on the aspect ratio of the Dock.
         """
-        #print self.name(), "setOrientation", o, force
+        # setOrientation may be called before the container is set in some cases
+        # (via resizeEvent), so there's no need to do anything here until called
+        # again by containerChanged
+        if self.container() is None:
+            return
+
         if o == 'auto' and self.autoOrient:
             if self.container().type() == 'tab':
                 o = 'horizontal'
@@ -146,22 +157,19 @@ class Dock(QtGui.QWidget, DockDrop):
             self.orientation = o
             self.label.setOrientation(o)
             self.updateStyle()
-        
+
     def updateStyle(self):
         ## updates orientation and appearance of title bar
-        #print self.name(), "update style:", self.orientation, self.moveLabel, self.label.isVisible()
         if self.labelHidden:
             self.widgetArea.setStyleSheet(self.nStyle)
         elif self.orientation == 'vertical':
             self.label.setOrientation('vertical')
             if self.moveLabel:
-                #print self.name(), "reclaim label"
                 self.topLayout.addWidget(self.label, 1, 0)
             self.widgetArea.setStyleSheet(self.vStyle)
         else:
             self.label.setOrientation('horizontal')
             if self.moveLabel:
-                #print self.name(), "reclaim label"
                 self.topLayout.addWidget(self.label, 0, 1)
             self.widgetArea.setStyleSheet(self.hStyle)
 
@@ -170,10 +178,7 @@ class Dock(QtGui.QWidget, DockDrop):
         self.resizeOverlay(self.size())
 
     def name(self):
-        return asUnicode(self.label.text())
-
-    def container(self):
-        return self._container
+        return self._name
 
     def addWidget(self, widget, row=None, col=0, rowspan=1, colspan=1):
         """
@@ -187,42 +192,50 @@ class Dock(QtGui.QWidget, DockDrop):
         self.layout.addWidget(widget, row, col, rowspan, colspan)
         self.raiseOverlay()
         
-        
     def startDrag(self):
         self.drag = QtGui.QDrag(self)
         mime = QtCore.QMimeData()
-        #mime.setPlainText("asd")
         self.drag.setMimeData(mime)
         self.widgetArea.setStyleSheet(self.dragStyle)
         self.update()
-        action = self.drag.exec_()
+        action = self.drag.exec_() if hasattr(self.drag, 'exec_') else self.drag.exec()
         self.updateStyle()
-        
+
     def float(self):
         self.area.floatDock(self)
             
+    def container(self):
+        return self._container
+
     def containerChanged(self, c):
-        #print self.name(), "container changed"
+        if self._container is not None:
+            # ask old container to close itself if it is no longer needed
+            self._container.apoptose()
         self._container = c
-        if c.type() != 'tab':
-            self.moveLabel = True
-            self.label.setDim(False)
+        if c is None:
+            self.area = None
         else:
-            self.moveLabel = False
-            
-        self.setOrientation(force=True)
-        
+            self.area = c.area
+            if c.type() != 'tab':
+                self.moveLabel = True
+                self.label.setDim(False)
+            else:
+                self.moveLabel = False
+                
+            self.setOrientation(force=True)
+
     def raiseDock(self):
         """If this Dock is stacked underneath others, raise it to the top."""
         self.container().raiseDock(self)
-        
 
     def close(self):
         """Remove this dock from the DockArea it lives inside."""
         self.setParent(None)
+        QtGui.QLabel.close(self.label)
         self.label.setParent(None)
         self._container.apoptose()
         self._container = None
+        self.sigClosed.emit(self)
 
     def __repr__(self):
         return "<Dock %s %s>" % (self.name(), self.stretch())
@@ -243,19 +256,20 @@ class Dock(QtGui.QWidget, DockDrop):
 
 
 class DockLabel(VerticalLabel):
-    
+
     sigClicked = QtCore.Signal(object, object)
     sigCloseClicked = QtCore.Signal()
-    
-    def __init__(self, text, dock, showCloseButton):
+
+    def __init__(self, text, dock, showCloseButton, fontSize):
         self.dim = False
         self.fixedWidth = False
+        self.fontSize = fontSize
         VerticalLabel.__init__(self, text, orientation='horizontal', forceWidth=False)
         self.setAlignment(QtCore.Qt.AlignTop|QtCore.Qt.AlignHCenter)
         self.dock = dock
         self.updateStyle()
         self.setAutoFillBackground(False)
-        self.startedDrag = False
+        self.mouseMoved = False
 
         self.closeButton = None
         if showCloseButton:
@@ -273,7 +287,7 @@ class DockLabel(VerticalLabel):
             fg = '#fff'
             bg = '#66c'
             border = '#55B'
-        
+
         if self.orientation == 'vertical':
             self.vStyle = """DockLabel {
                 background-color : %s;
@@ -286,7 +300,8 @@ class DockLabel(VerticalLabel):
                 border-right: 2px solid %s;
                 padding-top: 3px;
                 padding-bottom: 3px;
-            }""" % (bg, fg, r, r, border)
+                font-size: %s;
+            }""" % (bg, fg, r, r, border, self.fontSize)
             self.setStyleSheet(self.vStyle)
         else:
             self.hStyle = """DockLabel {
@@ -300,38 +315,41 @@ class DockLabel(VerticalLabel):
                 border-bottom: 2px solid %s;
                 padding-left: 3px;
                 padding-right: 3px;
-            }""" % (bg, fg, r, r, border)
+                font-size: %s;
+            }""" % (bg, fg, r, r, border, self.fontSize)
             self.setStyleSheet(self.hStyle)
 
     def setDim(self, d):
         if self.dim != d:
             self.dim = d
             self.updateStyle()
-    
+
     def setOrientation(self, o):
         VerticalLabel.setOrientation(self, o)
         self.updateStyle()
 
     def mousePressEvent(self, ev):
-        if ev.button() == QtCore.Qt.LeftButton:
-            self.pressPos = ev.pos()
-            self.startedDrag = False
-            ev.accept()
-        
+        self.pressPos = ev.localPos()
+        self.mouseMoved = False
+        ev.accept()
+
     def mouseMoveEvent(self, ev):
-        if not self.startedDrag and (ev.pos() - self.pressPos).manhattanLength() > QtGui.QApplication.startDragDistance():
+        if not self.mouseMoved:
+            self.mouseMoved = (ev.localPos() - self.pressPos).manhattanLength() > QtGui.QApplication.startDragDistance()
+
+        if self.mouseMoved and ev.buttons() == QtCore.Qt.LeftButton:
             self.dock.startDrag()
         ev.accept()
-            
+
     def mouseReleaseEvent(self, ev):
-        if not self.startedDrag:
-            self.sigClicked.emit(self, ev)
         ev.accept()
-        
+        if not self.mouseMoved:
+            self.sigClicked.emit(self, ev)
+
     def mouseDoubleClickEvent(self, ev):
         if ev.button() == QtCore.Qt.LeftButton:
             self.dock.float()
-            
+
     def resizeEvent (self, ev):
         if self.closeButton:
             if self.orientation == 'vertical':
