@@ -1,15 +1,18 @@
 # -*- coding: utf-8 -*-
+import math
+
 from .GraphicsWidget import GraphicsWidget
 from .LabelItem import LabelItem
 from ..Qt import QtGui, QtCore
 from .. import functions as fn
+from ..icons import invisibleEye
 from ..Point import Point
 from .ScatterPlotItem import ScatterPlotItem, drawSymbol
 from .PlotDataItem import PlotDataItem
 from .GraphicsWidgetAnchor import GraphicsWidgetAnchor
 from .BarGraphItem import BarGraphItem
 
-__all__ = ['LegendItem']
+__all__ = ['LegendItem', 'ItemSample']
 
 
 class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
@@ -29,7 +32,7 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
 
     def __init__(self, size=None, offset=None, horSpacing=25, verSpacing=0,
                  pen=None, brush=None, labelTextColor=None, frame=True,
-                 labelTextSize='9pt', rowCount=1, colCount=1, **kwargs):
+                 labelTextSize='9pt', colCount=1, sampleType=None, **kwargs):
         """
         ==============  ===============================================================
         **Arguments:**
@@ -52,6 +55,11 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
                         accepted by :func:`mkPen <pyqtgraph.mkPen>` is allowed.
         labelTextSize   Size to use when drawing legend text. Accepts CSS style
                         string arguments, e.g. '9pt'.
+        colCount        Specifies the integer number of columns that the legend should
+                        be divided into. The number of rows will be calculated
+                        based on this argument. This is useful for plots with many
+                        curves displayed simultaneously. Default: 1 column.
+        sampleType      Customizes the item sample class of the `LegendItem`.
         ==============  ===============================================================
 
         """
@@ -68,11 +76,18 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
         self.offset = offset
         self.frame = frame
         self.columnCount = colCount
-        self.rowCount = rowCount
-        self.curRow = 0
+        self.rowCount = 1
         if size is not None:
             self.setGeometry(QtCore.QRectF(0, 0, self.size[0], self.size[1]))
 
+        if sampleType is not None:
+            if not issubclass(sampleType, GraphicsWidget):
+                raise RuntimeError("Only classes of type `GraphicsWidgets` "
+                                   "are allowed as `sampleType`")
+            self.sampleType = sampleType
+        else:
+            self.sampleType = ItemSample
+
         self.opts = {
             'pen': fn.mkPen(pen),
             'brush': fn.mkBrush(brush),
@@ -80,9 +95,26 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
             'labelTextSize': labelTextSize,
             'offset': offset,
         }
-
         self.opts.update(kwargs)
 
+    def setSampleType(self, sample):
+        """Set the new sample item claspes"""
+        if sample is self.sampleType:
+            return
+
+        # Clear the legend, but before create a list of items
+        items = list(self.items)
+        self.sampleType = sample
+        self.clear()
+
+        # Refill the legend with the item list and new sample item
+        for sample, label in items:
+            plot_item = sample.item
+            plot_name = label.text
+            self.addItem(plot_item, plot_name)
+
+        self.updateSize()
+
     def offset(self):
         """Get the offset position relative to the parent."""
         return self.opts['offset']
@@ -183,10 +215,10 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
         """
         label = LabelItem(name, color=self.opts['labelTextColor'],
                           justify='left', size=self.opts['labelTextSize'])
-        if isinstance(item, ItemSample):
+        if isinstance(item, self.sampleType):
             sample = item
         else:
-            sample = ItemSample(item)
+            sample = self.sampleType(item)
         self.items.append((sample, label))
         self._addItemToLayout(sample, label)
         self.updateSize()
@@ -203,19 +235,23 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
                 # FIND RIGHT COLUMN
                 if not self.layout.itemAt(row, col):
                     break
-            if col + 2 == nCol:
-                # MAKE NEW ROW
-                col = 0
-                row += 1
+            else:
+                if col + 2 == nCol:
+                    # MAKE NEW ROW
+                    col = 0
+                    row += 1
         self.layout.addItem(sample, row, col)
         self.layout.addItem(label, row, col + 1)
+        # Keep rowCount in sync with the number of rows if items are added
+        self.rowCount = max(self.rowCount, row + 1)
 
     def setColumnCount(self, columnCount):
         """change the orientation of all items of the legend
         """
         if columnCount != self.columnCount:
             self.columnCount = columnCount
-            self.rowCount = int(len(self.items) / columnCount)
+
+            self.rowCount = math.ceil(len(self.items) / columnCount)
             for i in range(self.layout.count() - 1, -1, -1):
                 self.layout.removeAt(i)  # clear layout
             for sample, label in self.items:
@@ -225,7 +261,7 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
     def getLabel(self, plotItem):
         """Return the labelItem inside the legend for a given plotItem
 
-        The label-text can be changed via labenItem.setText
+        The label-text can be changed via labelItem.setText
         """
         out = [(it, lab) for it, lab in self.items if it.item == plotItem]
         try:
@@ -269,13 +305,13 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
         width = 0
         for row in range(self.layout.rowCount()):
             row_height = 0
-            col_witdh = 0
+            col_width = 0
             for col in range(self.layout.columnCount()):
                 item = self.layout.itemAt(row, col)
                 if item:
-                    col_witdh += item.width() + 3
+                    col_width += item.width() + 3
                     row_height = max(row_height, item.height())
-            width = max(width, col_witdh)
+            width = max(width, col_width)
             height += row_height
         self.setGeometry(0, 0, width, height)
         return
@@ -301,11 +337,8 @@ class LegendItem(GraphicsWidget, GraphicsWidgetAnchor):
 
 class ItemSample(GraphicsWidget):
     """Class responsible for drawing a single item in a LegendItem (sans label)
-
-    This may be subclassed to draw custom graphics in a Legend.
     """
 
-    # TODO: make this more generic; let items decide how it should be look.
     def __init__(self, item):
         GraphicsWidget.__init__(self)
         self.item = item
@@ -315,10 +348,15 @@ class ItemSample(GraphicsWidget):
 
     def paint(self, p, *args):
         opts = self.item.opts
-
         if opts.get('antialias'):
             p.setRenderHint(p.Antialiasing)
 
+        visible = self.item.isVisible()
+        if not visible:
+            icon = invisibleEye.qicon
+            p.drawPixmap(QtCore.QPoint(1, 1), icon.pixmap(18, 18))
+            return
+
         if not isinstance(self.item, ScatterPlotItem):
             p.setPen(fn.mkPen(opts['pen']))
             p.drawLine(0, 11, 20, 11)
@@ -326,7 +364,7 @@ class ItemSample(GraphicsWidget):
             if (opts.get('fillLevel', None) is not None and
                     opts.get('fillBrush', None) is not None):
                 p.setBrush(fn.mkBrush(opts['fillBrush']))
-                p.setPen(fn.mkPen(opts['fillBrush']))
+                p.setPen(fn.mkPen(opts['pen']))
                 p.drawPolygon(QtGui.QPolygonF(
                     [QtCore.QPointF(2, 18), QtCore.QPointF(18, 2),
                      QtCore.QPointF(18, 18)]))
@@ -342,3 +380,13 @@ class ItemSample(GraphicsWidget):
         if isinstance(self.item, BarGraphItem):
             p.setBrush(fn.mkBrush(opts['brush']))
             p.drawRect(QtCore.QRectF(2, 2, 18, 18))
+
+    def mouseClickEvent(self, event):
+        """Use the mouseClick event to toggle the visibility of the plotItem
+        """
+        if event.button() == QtCore.Qt.LeftButton:
+            visible = self.item.isVisible()
+            self.item.setVisible(not visible)
+
+        event.accept()
+        self.update()
