@@ -1,16 +1,15 @@
+# -*- coding: utf-8 -*-
 from ...Qt import QtCore, QtGui, QT_LIB
 from ...python2_3 import asUnicode
 from ...WidgetGroup import WidgetGroup
 
-if QT_LIB == 'PyQt4':
-    from .axisCtrlTemplate_pyqt import Ui_Form as AxisCtrlTemplate
-elif QT_LIB == 'PySide':
-    from .axisCtrlTemplate_pyside import Ui_Form as AxisCtrlTemplate
-elif QT_LIB == 'PyQt5':
-    from .axisCtrlTemplate_pyqt5 import Ui_Form as AxisCtrlTemplate
-    
+import importlib
+ui_template = importlib.import_module(
+    f'.axisCtrlTemplate_{QT_LIB.lower()}', package=__package__)
+
 import weakref 
 
+translate = QtCore.QCoreApplication.translate
 class ViewBoxMenu(QtGui.QMenu):
     def __init__(self, view):
         QtGui.QMenu.__init__(self)
@@ -19,8 +18,8 @@ class ViewBoxMenu(QtGui.QMenu):
         self.valid = False  ## tells us whether the ui needs to be updated
         self.viewMap = weakref.WeakValueDictionary()  ## weakrefs to all views listed in the link combos
 
-        self.setTitle("ViewBox options")
-        self.viewAll = QtGui.QAction("View All", self)
+        self.setTitle(translate("ViewBox", "ViewBox options"))
+        self.viewAll = QtGui.QAction(translate("ViewBox", "View All"), self)
         self.viewAll.triggered.connect(self.autoRange)
         self.addAction(self.viewAll)
         
@@ -30,9 +29,9 @@ class ViewBoxMenu(QtGui.QMenu):
         self.dv = QtGui.QDoubleValidator(self)
         for axis in 'XY':
             m = QtGui.QMenu()
-            m.setTitle("%s Axis" % axis)
+            m.setTitle(f"{axis} {translate('ViewBox', 'axis')}")
             w = QtGui.QWidget()
-            ui = AxisCtrlTemplate()
+            ui = ui_template.Ui_Form()
             ui.setupUi(w)
             a = QtGui.QWidgetAction(self)
             a.setDefaultWidget(w)
@@ -46,8 +45,8 @@ class ViewBoxMenu(QtGui.QMenu):
             connects = [
                 (ui.mouseCheck.toggled, 'MouseToggled'),
                 (ui.manualRadio.clicked, 'ManualClicked'),
-                (ui.minText.editingFinished, 'MinTextChanged'),
-                (ui.maxText.editingFinished, 'MaxTextChanged'),
+                (ui.minText.editingFinished, 'RangeTextChanged'),
+                (ui.maxText.editingFinished, 'RangeTextChanged'),
                 (ui.autoRadio.clicked, 'AutoClicked'),
                 (ui.autoPercentSpin.valueChanged, 'AutoSpinChanged'),
                 (ui.linkCombo.currentIndexChanged, 'LinkComboChanged'),
@@ -65,15 +64,15 @@ class ViewBoxMenu(QtGui.QMenu):
         #self.setExportMethods(view.exportMethods)
         #self.addMenu(self.export)
         
-        self.leftMenu = QtGui.QMenu("Mouse Mode")
+        self.leftMenu = QtGui.QMenu(translate("ViewBox", "Mouse Mode"))
         group = QtGui.QActionGroup(self)
         
         # This does not work! QAction _must_ be initialized with a permanent 
         # object as the parent or else it may be collected prematurely.
         #pan = self.leftMenu.addAction("3 button", self.set3ButtonMode)
         #zoom = self.leftMenu.addAction("1 button", self.set1ButtonMode)
-        pan = QtGui.QAction("3 button", self.leftMenu)
-        zoom = QtGui.QAction("1 button", self.leftMenu)
+        pan = QtGui.QAction(translate("ViewBox", "3 button"), self.leftMenu)
+        zoom = QtGui.QAction(translate("ViewBox", "1 button"), self.leftMenu)
         self.leftMenu.addAction(pan)
         self.leftMenu.addAction(zoom)
         pan.triggered.connect(self.set3ButtonMode)
@@ -160,14 +159,10 @@ class ViewBoxMenu(QtGui.QMenu):
     def xManualClicked(self):
         self.view().enableAutoRange(ViewBox.XAxis, False)
         
-    def xMinTextChanged(self):
+    def xRangeTextChanged(self):
         self.ctrl[0].manualRadio.setChecked(True)
-        self.view().setXRange(float(self.ctrl[0].minText.text()), float(self.ctrl[0].maxText.text()), padding=0)
+        self.view().setXRange(*self._validateRangeText(0), padding=0)
 
-    def xMaxTextChanged(self):
-        self.ctrl[0].manualRadio.setChecked(True)
-        self.view().setXRange(float(self.ctrl[0].minText.text()), float(self.ctrl[0].maxText.text()), padding=0)
-        
     def xAutoClicked(self):
         val = self.ctrl[0].autoPercentSpin.value() * 0.01
         self.view().enableAutoRange(ViewBox.XAxis, val)
@@ -192,13 +187,9 @@ class ViewBoxMenu(QtGui.QMenu):
     def yManualClicked(self):
         self.view().enableAutoRange(ViewBox.YAxis, False)
         
-    def yMinTextChanged(self):
+    def yRangeTextChanged(self):
         self.ctrl[1].manualRadio.setChecked(True)
-        self.view().setYRange(float(self.ctrl[1].minText.text()), float(self.ctrl[1].maxText.text()), padding=0)
-        
-    def yMaxTextChanged(self):
-        self.ctrl[1].manualRadio.setChecked(True)
-        self.view().setYRange(float(self.ctrl[1].minText.text()), float(self.ctrl[1].maxText.text()), padding=0)
+        self.view().setYRange(*self._validateRangeText(1), padding=0)
         
     def yAutoClicked(self):
         val = self.ctrl[1].autoPercentSpin.value() * 0.01
@@ -263,6 +254,20 @@ class ViewBoxMenu(QtGui.QMenu):
             if changed:
                 c.setCurrentIndex(0)
                 c.currentIndexChanged.emit(c.currentIndex())
+
+    def _validateRangeText(self, axis):
+        """Validate range text inputs. Return current value(s) if invalid."""
+        inputs = (self.ctrl[axis].minText.text(),
+                  self.ctrl[axis].maxText.text())
+        vals = self.view().viewRange()[axis]
+        for i, text in enumerate(inputs):
+            try:
+                vals[i] = float(text)
+            except ValueError:
+                # could not convert string to float
+                pass
+        return vals
+
         
 from .ViewBox import ViewBox
         
