@@ -1,13 +1,21 @@
+# -*- coding: utf-8 -*-
+from .. import functions as fn
 from ..Qt import QtGui, QtCore
 import os, weakref, re
-from ..pgcollections import OrderedDict
-from ..python2_3 import asUnicode
+from collections import OrderedDict
+from ..python2_3 import asUnicode, basestring
 from .ParameterItem import ParameterItem
+import warnings
 
 PARAM_TYPES = {}
 PARAM_NAMES = {}
 
 def registerParameterType(name, cls, override=False):
+    """Register a parameter type in the parametertree system.
+
+    This enables construction of custom Parameter classes by name in
+    :meth:`~pyqtgraph.parametertree.Parameter.create`.
+    """
     global PARAM_TYPES
     if name in PARAM_TYPES and not override:
         raise Exception("Parameter type '%s' already exists (use override=True to replace)" % name)
@@ -55,6 +63,7 @@ class Parameter(QtCore.QObject):
     sigDefaultChanged(self, default)     Emitted when this parameter's default value has changed
     sigNameChanged(self, name)           Emitted when this parameter's name has changed
     sigOptionsChanged(self, opts)        Emitted when any of this parameter's options have changed
+    sigContextMenu(self, name)           Emitted when a context menu was clicked
     ===================================  =========================================================
     """
     ## name, type, limits, etc.
@@ -81,7 +90,8 @@ class Parameter(QtCore.QObject):
     ## (but only if monitorChildren() is called)
     sigTreeStateChanged = QtCore.Signal(object, object)  # self, changes
                                                          # changes = [(param, change, info), ...]
-    
+    sigContextMenu = QtCore.Signal(object, object)       # self, name
+
     # bad planning.
     #def __new__(cls, *args, **opts):
         #try:
@@ -135,9 +145,12 @@ class Parameter(QtCore.QObject):
                                      (default=False)
         removable                    If True, the user may remove this Parameter.
                                      (default=False)
-        expanded                     If True, the Parameter will appear expanded when
-                                     displayed in a ParameterTree (its children will be
-                                     visible). (default=True)
+        expanded                     If True, the Parameter will initially be expanded in
+                                     ParameterTrees: Its children will be visible.
+                                     (default=True)
+        syncExpanded                 If True, the `expanded` state of this Parameter is
+                                     synchronized with all ParameterTrees it is displayed in.
+                                     (default=False)
         title                        (str or None) If specified, then the parameter will be 
                                      displayed to the user using this string as its name. 
                                      However, the parameter will still be referred to 
@@ -159,10 +172,15 @@ class Parameter(QtCore.QObject):
             'removable': False,
             'strictNaming': False,  # forces name to be usable as a python variable
             'expanded': True,
+            'syncExpanded': False,
             'title': None,
             #'limits': None,  ## This is a bad plan--each parameter type may have a different data type for limits.
         }
+        value = opts.get('value', None)
+        name = opts.get('name', None)
         self.opts.update(opts)
+        self.opts['value'] = None  # will be set later.
+        self.opts['name'] = None
         
         self.childs = []
         self.names = {}   ## map name:child
@@ -172,27 +190,30 @@ class Parameter(QtCore.QObject):
         self.blockTreeChangeEmit = 0
         #self.monitoringChildren = False  ## prevent calling monitorChildren more than once
         
-        if 'value' not in self.opts:
-            self.opts['value'] = None
-        
-        if 'name' not in self.opts or not isinstance(self.opts['name'], basestring):
+        if not isinstance(name, basestring):
             raise Exception("Parameter must have a string name specified in opts.")
-        self.setName(opts['name'])
+        self.setName(name)
         
-        self.addChildren(self.opts.get('children', []))
-            
-        if 'value' in self.opts and 'default' not in self.opts:
-            self.opts['default'] = self.opts['value']
+        self.addChildren(self.opts.pop('children', []))
+        
+        if value is not None:
+            self.setValue(value)
+
+        if 'default' not in self.opts:
+            self.opts['default'] = None
+            self.setDefault(self.opts['value'])
     
         ## Connect all state changed signals to the general sigStateChanged
-        self.sigValueChanged.connect(lambda param, data: self.emitStateChanged('value', data))
-        self.sigChildAdded.connect(lambda param, *data: self.emitStateChanged('childAdded', data))
-        self.sigChildRemoved.connect(lambda param, data: self.emitStateChanged('childRemoved', data))
-        self.sigParentChanged.connect(lambda param, data: self.emitStateChanged('parent', data))
-        self.sigLimitsChanged.connect(lambda param, data: self.emitStateChanged('limits', data))
-        self.sigDefaultChanged.connect(lambda param, data: self.emitStateChanged('default', data))
-        self.sigNameChanged.connect(lambda param, data: self.emitStateChanged('name', data))
-        self.sigOptionsChanged.connect(lambda param, data: self.emitStateChanged('options', data))
+        self.sigValueChanged.connect(self._emitValueChanged)
+        self.sigChildAdded.connect(self._emitChildAddedChanged)
+        self.sigChildRemoved.connect(self._emitChildRemovedChanged)
+        self.sigParentChanged.connect(self._emitParentChanged)
+        self.sigLimitsChanged.connect(self._emitLimitsChanged)
+        self.sigDefaultChanged.connect(self._emitDefaultChanged)
+        self.sigNameChanged.connect(self._emitNameChanged)
+        self.sigOptionsChanged.connect(self._emitOptionsChanged)
+        self.sigContextMenu.connect(self._emitContextMenuChanged)
+
         
         #self.watchParam(self)  ## emit treechange signals if our own state changes
         
@@ -200,6 +221,20 @@ class Parameter(QtCore.QObject):
         """Return the name of this Parameter."""
         return self.opts['name']
 
+    def title(self):
+        """Return the title of this Parameter.
+        
+        By default, the title is the same as the name unless it has been explicitly specified
+        otherwise."""
+        title = self.opts.get('title', None)
+        if title is None:
+            title = self.name()
+        return title
+
+    def contextMenu(self, name):
+        """"A context menu entry was clicked"""
+        self.sigContextMenu.emit(self, name)
+
     def setName(self, name):
         """Attempt to change the name of this parameter; return the actual name. 
         (The parameter may reject the name change or automatically pick a different name)"""
@@ -255,15 +290,19 @@ class Parameter(QtCore.QObject):
         try:
             if blockSignal is not None:
                 self.sigValueChanged.disconnect(blockSignal)
-            if self.opts['value'] == value:
+            value = self._interpretValue(value)
+            if fn.eq(self.opts['value'], value):
                 return value
             self.opts['value'] = value
-            self.sigValueChanged.emit(self, value)
+            self.sigValueChanged.emit(self, value)  # value might change after signal is received by tree item
         finally:
             if blockSignal is not None:
                 self.sigValueChanged.connect(blockSignal)
             
-        return value
+        return self.opts['value']
+
+    def _interpretValue(self, v):
+        return v
 
     def value(self):
         """
@@ -312,7 +351,8 @@ class Parameter(QtCore.QObject):
         If blockSignals is True, no signals will be emitted until the tree has been completely restored. 
         This prevents signal handlers from responding to a partially-rebuilt network.
         """
-        childState = state.get('children', [])
+        state = state.copy()
+        childState = state.pop('children', [])
         
         ## list of children may be stored either as list or dict.
         if isinstance(childState, dict):
@@ -397,7 +437,7 @@ class Parameter(QtCore.QObject):
 
     def hasDefault(self):
         """Returns True if this parameter has a default value."""
-        return 'default' in self.opts
+        return self.opts['default'] is not None
         
     def valueIsDefault(self):
         """Returns True if this parameter's value is equal to the default value."""
@@ -442,7 +482,7 @@ class Parameter(QtCore.QObject):
         Set any arbitrary options on this parameter.
         The exact behavior of this function will depend on the parameter type, but
         most parameters will accept a common set of options: value, name, limits,
-        default, readonly, removable, renamable, visible, enabled, and expanded.
+        default, readonly, removable, renamable, visible, enabled, expanded and syncExpanded.
         
         See :func:`Parameter.__init__ <pyqtgraph.parametertree.Parameter.__init__>`
         for more information on default options.
@@ -472,6 +512,33 @@ class Parameter(QtCore.QObject):
         self.treeStateChanges.append((self, changeDesc, data))
         self.emitTreeChanges()
 
+    def _emitValueChanged(self, param, data):
+        self.emitStateChanged("value", data)
+
+    def _emitChildAddedChanged(self, param, *data):
+        self.emitStateChanged("childAdded", data)
+
+    def _emitChildRemovedChanged(self, param, data):
+        self.emitStateChanged("childRemoved", data)
+
+    def _emitParentChanged(self, param, data):
+        self.emitStateChanged("parent", data)
+
+    def _emitLimitsChanged(self, param, data):
+        self.emitStateChanged("limits", data)
+
+    def _emitDefaultChanged(self, param, data):
+        self.emitStateChanged("default", data)
+
+    def _emitNameChanged(self, param, data):
+        self.emitStateChanged("name", data)
+
+    def _emitOptionsChanged(self, param, data):
+        self.emitStateChanged("options", data)
+
+    def _emitContextMenuChanged(self, param, data):
+        self.emitStateChanged("contextMenu", data)
+
     def makeTreeItem(self, depth):
         """
         Return a TreeWidgetItem suitable for displaying/controlling the content of 
@@ -548,8 +615,8 @@ class Parameter(QtCore.QObject):
             self.childs.insert(pos, child)
             
             child.parentChanged(self)
-            self.sigChildAdded.emit(self, child, pos)
             child.sigTreeStateChanged.connect(self.treeStateChanged)
+            self.sigChildAdded.emit(self, child, pos)
         return child
         
     def removeChild(self, child):
@@ -560,11 +627,11 @@ class Parameter(QtCore.QObject):
         del self.names[name]
         self.childs.pop(self.childs.index(child))
         child.parentChanged(None)
-        self.sigChildRemoved.emit(self, child)
         try:
             child.sigTreeStateChanged.disconnect(self.treeStateChanged)
         except (TypeError, RuntimeError):  ## already disconnected
             pass
+        self.sigChildRemoved.emit(self, child)
 
     def clearChildren(self):
         """Remove all child parameters."""
@@ -601,7 +668,7 @@ class Parameter(QtCore.QObject):
 
     def incrementName(self, name):
         ## return an unused name by adding a number to the name given
-        base, num = re.match('(.*)(\d*)', name).groups()
+        base, num = re.match(r'(.*)(\d*)', name).groups()
         numLen = len(num)
         if numLen == 0:
             num = 2
@@ -638,22 +705,26 @@ class Parameter(QtCore.QObject):
             names = (names,)
         return self.param(*names).setValue(value)
 
+    def keys(self):
+        return self.names
+
     def child(self, *names):
         """Return a child parameter. 
         Accepts the name of the child or a tuple (path, to, child)
 
-        Added in version 0.9.9. Ealier versions used the 'param' method, which is still
-        implemented for backward compatibility."""
+        Added in version 0.9.9. Earlier versions used the 'param' method, which is still
+        implemented for backward compatibility.
+        """
         try:
             param = self.names[names[0]]
         except KeyError:
-            raise Exception("Parameter %s has no child named %s" % (self.name(), names[0]))
+            raise KeyError("Parameter %s has no child named %s" % (self.name(), names[0]))
         
         if len(names) > 1:
-            return param.param(*names[1:])
+            return param.child(*names[1:])
         else:
             return param
-        
+
     def param(self, *names):
         # for backward compatibility.
         return self.child(*names)
@@ -664,7 +735,11 @@ class Parameter(QtCore.QObject):
     def __getattr__(self, attr):
         ## Leaving this undocumented because I might like to remove it in the future..
         #print type(self), attr
-        
+        warnings.warn(
+            'Use of Parameter.subParam is deprecated and will be removed in 0.13 '
+            'Use Parameter.param(name) instead.',
+            DeprecationWarning, stacklevel=2
+        )          
         if 'names' not in self.__dict__:
             raise AttributeError(attr)
         if attr in self.names:
